! Copyright (c) 1987 Edinburgh Portable Compilers Ltd. All Rights Reserved.
!*
!*                                                                ft_intel.1.inc!* modified 14/10/92 to get correct values for backend(pds)
!*                                                                ft_sparc3.1
!* modified 27/07/92 - for INTEL port
!*                                                                ft_sparc3
!* modified 13/03/92 - Conform to new standard format for banners.
!*                                                                ft_sparc2
!* modified 08/07/91 - Made specific version for C++. (rt)
!*                                                                ft_sparc1
!* modified 21/05/91 - Made SPARC specific version in merge of M88K and
!*                     SPARC front ends. (rt)
!*                                                                ftnht8
!* modified 08/02/91 - Made changes for port to M88K based mosquito. Added
!*                     UNIXDebug constant from cf_m88kopt1. Many of the
!*                     constants in this module are not actually used.
!*                     This module should be tidied up at some time in the
!*                     and incorporated into the cf_<sys>opt<n> file. (rt)
!*                                                                ftnht7.7
!* modified 09/04/91 - Changed compiler to version 2.1 and added new
!*                     copyright string. (rt)
!*                                                                ftnht7.6un
!* modified 18/06/90 - definition of VAX removed as it clashes with field
!*                     in gccompile53.1.i
!* modified 05/04/90 - add constinteger for Aligndata              ftnht7.6
!* modified 13/03/90 - add constinteger for Locust                 ftnht7.5
!* modified 18/01/90 - define Allowparallel                        ftnht7.4
!*                   - define Directcomplex
!*                   - define Fregopt
!* modified 12/01/90 - define ALIGN8 for REAL*8 alignment reqmnts  ftnht7.3
!* modified 17/11/89 - define Emachine                             ftnht7.2
!* modified 30/09/89 - define sparc,m88000,i860,paramdirection     ftnht7.1
!* modified 05/09/89 - Add constinteger for modula
!*                   - add constinteger for compiler
!*                   - put reldate in ftnht instead of mfcompile
!* modified 01/05/89 - define TrueValue 
!* modified 25/03/89 - define Allowintprc to govern the legality of
!*                     internal procedures
!* modified 06/01/89 - Remove flag NP2
!* modified 12/12/88 - Set NP2=1 Goulddivnewbr=1 for NP2
!* modified 10/12/88 - NP2 defined as a Boolean
!* modified 25/11/88 - define Goulddivnewbr
!*                   - define Scaleopt1,Allowcase,IEEE
!* modified 06/05/88 - define NP2
!* modified 21/03/88 - define Allowgldfns
!* modified 04/03/88 - more definitions added
!* modified 26/01/88 - define Numoptregs
!* modified 25/01/88 - define Bytesreversed
!*                   - define BSSavailable
!*                   - define Usechipfns
!*                   - define Allowvaxnames
!*                   - define Allow99cont
!*                   - define Realsubscripts
!*                   - define AllowI8decl
!* modified 13/09/87 - define Evalrealslong
!*                   - define AllowI8
!* modified 07/09/87 - define Array temps on stack
!*
%constinteger ICL2900 = 1
%constinteger PERQPNX = 2
%constinteger ACCENT  = 3
%constinteger IBM     = 4
!*
%constinteger M68000   =6
%constinteger Sun3     =6
%constinteger INTEL    =7
%constinteger LATTICE  =8
%constinteger VNS      =9
%constinteger GOULD    =10
%constinteger WWC      =11
%constinteger WCW      =11
%constinteger DRS      =11
%constinteger AMDAHL   =14
%constinteger Sequent  =15
%constinteger SPARC    =16
%constinteger M88000   =17
%constinteger M88K     =17

%constinteger i860     =18
%constinteger Locust   =SPARC
%constinteger Mosquito =M88K
%constinteger Tigermoth=M88K
%constinteger Aphid    =SPARC
%constinteger M88110  =20
%constinteger M88100  =21
%constinteger M88100forM88110 =22
%constinteger RS6000=23
%constinteger DRS486   =24
%constinteger PTX     =25
%constinteger RTMX     =26
%constinteger IntelSolaris=27
!*
%constinteger POSITIVE =  1
%constinteger NEGATIVE = -1
%constinteger TRUE     =  1
%constinteger FALSE    =  0
!*
%constinteger HOST    = INTEL
%constinteger TARGET  = INTEL
%constinteger Env     = IntelSolaris
!*
{ UNIX debugger support constants }
%constinteger UD UNIMP = 0  { No debugger support provided              }
%constinteger UD SDB   = 1  { Support provided for SDB                  }
%constinteger UD DBX   = 2  { Support provided for DBX                  }
%constinteger UD EDB   = 3  { Support provided for proprietary debugger }
!*
{ UNIX debugger support in this environment }
%constinteger UNIXDebug = UD UNIMP    { No debugger support on LOCUST }

!*
%constinteger UnixType = 5
!* Unix type is set to five for LOCUST
%constinteger Unix2   = False
%constinteger Unix4   = False
%constinteger Unix5d4 = False
%constinteger Unix5   = True
!* Why we have to respecify the Unix type I do not know.  It seems
!* illogical, but that is how fntht7.6un.inc works
%constinteger STACK DIRECTION = NEGATIVE
%constinteger PARAM DIRECTION = POSITIVE
!*
%constinteger ALIGN8  = (1 << GOULD) ! (1 << SPARC) ! (1 << M88000)
!*
%constinteger Concept = 0        {}
%constinteger NP1     = 1        {only applicable if host/target=Gould}
%constinteger Cpu     = Concept  {}
%constinteger Goulddivnewbr = 0  { 0 not available  1 available }
!*
%constinteger Allowvector = 0            { 0  No   1  Yes }
%constinteger Allowparallel = 0          { 0  No   1  Yes }
%constinteger Emachine      = 4
%constinteger Directcomplex = 0  { 0 No (indirect operands)  1 Yes (direct)  }
!*
%constinteger Array temps on stack = 0   { 0  No   1  Yes }
%constinteger AllowI8 = 0                { 0  No   1  Yes }
%constinteger Evalrealslong = 0          { 0  No   1  Yes }
%constinteger Bytesreversed = 0          { 0  No   1  Yes }
%constinteger BSSavailable  = 1          { 0  No   1  Yes }
%constinteger Allowvaxnames = 1          { 0  No   1  Yes }
%constinteger Allow99cont   = 1          { 0  No   1  Yes }
%constinteger Realsubscripts= 1          { 0  No   1  Yes }
%constinteger AllowI8decl   = 0          { 0  No   1  Yes }
%constinteger AllowI1decl   = 0          { 0  No   1  Yes }
%constinteger Allowfind     = 1          { 0  No   1  Yes   definefile,find}
%constinteger VAXparam      = 1          { 0  No   1  Yes }
%constinteger Bytesigned    = 1          { 0  No   1  Yes }
%constinteger FullVAX       = 1          { 0  No   1  Yes }
%constinteger Allowgldfns   = 0          { 0  No   1  Yes }
%constinteger Allowcase     = 0
%constinteger Numoptregs    = 1
%constinteger Allowintprc   = 0          {0   No   1  Yes }
%constinteger IEEE          = 1
%constinteger Scaleopt1     = 1
%constinteger Aligndata     = 1 { 1 add filler bytes in common}
%constinteger OCScompliant  = 0  { 0 No  1 Yes }
!*
%constinteger TrueValue     = 1          {usually 1, but x'ff' on PN and MPX}
!*
%constinteger UTX     = 0        {}
%constinteger MPX     = 1        {only applicable if host/target=Gould}
%constinteger Opsys   = UTX      {}
!*
%constinteger M68881     = 1
%constinteger Intel287   = 2
%constinteger Intel387   = 3
%constinteger Weitek     = 4
%constinteger Usechipfns = Intel387         {1 if none available}
!*
%constinteger Fregopt = 1                {0 not available  1 available}
!*
%constinteger full387 = 0                {applicable only on Intel}
!*
%constinteger IMP     =  1
%constinteger FORTRAN =  2
%constinteger CCOMP   = 11
%constinteger PASCAL  = 14
%constinteger MODULA  = 15

%constinteger Compiler = IMP

%constinteger ISCOFF = 0
%constinteger ISELF = 1
%constinteger ISBSD = 2
%constinteger ISBSDOMRON = 3
%constinteger Object Format = ISBSD
%conststring (60) EPCCversion = %c
"EPC C++ Version 2.1.5 (beta)                              
"

%conststring (60) EPCCcopyright = %c
"Copyright (c) 1992 EPCL. All Rights Reserved. 
"

%endoffile
