#include <perms.h>

                                                                //      1  ! 21Nov93 Extended to cope with PENTIUM and old code gen
                                                                //      2  !         by use of byte 1 of word4 in the gla(options word)
                                                                //      3     !Modified    21/September/88    15:00                               DELLDIAG8
                                                                //      4  
                                                                //      5  !*****************************************************
                                                                //      6  !*
                                                                //      7  !* DRS/NX  NDIAG - IMP monitors and trapped errors
                                                                //      8  !*                 come here. This routine cycles
                                                                //      9  !*                 down the stack, calling a lang.
                                                                //     10  !*                 dept. routine to analyse each frame
                                                                //     11  !*
                                                                //     12  !*****************************************************
                                                                //     13  
                                                                //     14  %include "ftnht.inc"
                                                                //      1  ! Copyright (c) 1988 Edinburgh Portable Compilers Ltd. All Rights Reserved.
                                                                //      2  !
                                                                //      3  %include "ft_intel.1.inc"
                                                                //      1  ! Copyright (c) 1987 Edinburgh Portable Compilers Ltd. All Rights Reserved.
                                                                //      2  !*
                                                                //      3  !*                                                                ft_intel.1.inc!* modified 14/10/92 to get correct values for backend(pds)
                                                                //      4  !*                                                                ft_sparc3.1
                                                                //      5  !* modified 27/07/92 - for INTEL port
                                                                //      6  !*                                                                ft_sparc3
                                                                //      7  !* modified 13/03/92 - Conform to new standard format for banners.
                                                                //      8  !*                                                                ft_sparc2
                                                                //      9  !* modified 08/07/91 - Made specific version for C++. (rt)
                                                                //     10  !*                                                                ft_sparc1
                                                                //     11  !* modified 21/05/91 - Made SPARC specific version in merge of M88K and
                                                                //     12  !*                     SPARC front ends. (rt)
                                                                //     13  !*                                                                ftnht8
                                                                //     14  !* modified 08/02/91 - Made changes for port to M88K based mosquito. Added
                                                                //     15  !*                     UNIXDebug constant from cf_m88kopt1. Many of the
                                                                //     16  !*                     constants in this module are not actually used.
                                                                //     17  !*                     This module should be tidied up at some time in the
                                                                //     18  !*                     and incorporated into the cf_<sys>opt<n> file. (rt)
                                                                //     19  !*                                                                ftnht7.7
                                                                //     20  !* modified 09/04/91 - Changed compiler to version 2.1 and added new
                                                                //     21  !*                     copyright string. (rt)
                                                                //     22  !*                                                                ftnht7.6un
                                                                //     23  !* modified 18/06/90 - definition of VAX removed as it clashes with field
                                                                //     24  !*                     in gccompile53.1.i
                                                                //     25  !* modified 05/04/90 - add constinteger for Aligndata              ftnht7.6
                                                                //     26  !* modified 13/03/90 - add constinteger for Locust                 ftnht7.5
                                                                //     27  !* modified 18/01/90 - define Allowparallel                        ftnht7.4
                                                                //     28  !*                   - define Directcomplex
                                                                //     29  !*                   - define Fregopt
                                                                //     30  !* modified 12/01/90 - define ALIGN8 for REAL*8 alignment reqmnts  ftnht7.3
                                                                //     31  !* modified 17/11/89 - define Emachine                             ftnht7.2
                                                                //     32  !* modified 30/09/89 - define sparc,m88000,i860,paramdirection     ftnht7.1
                                                                //     33  !* modified 05/09/89 - Add constinteger for modula
                                                                //     34  !*                   - add constinteger for compiler
                                                                //     35  !*                   - put reldate in ftnht instead of mfcompile
                                                                //     36  !* modified 01/05/89 - define TrueValue 
                                                                //     37  !* modified 25/03/89 - define Allowintprc to govern the legality of
                                                                //     38  !*                     internal procedures
                                                                //     39  !* modified 06/01/89 - Remove flag NP2
                                                                //     40  !* modified 12/12/88 - Set NP2=1 Goulddivnewbr=1 for NP2
                                                                //     41  !* modified 10/12/88 - NP2 defined as a Boolean
                                                                //     42  !* modified 25/11/88 - define Goulddivnewbr
                                                                //     43  !*                   - define Scaleopt1,Allowcase,IEEE
                                                                //     44  !* modified 06/05/88 - define NP2
                                                                //     45  !* modified 21/03/88 - define Allowgldfns
                                                                //     46  !* modified 04/03/88 - more definitions added
                                                                //     47  !* modified 26/01/88 - define Numoptregs
                                                                //     48  !* modified 25/01/88 - define Bytesreversed
                                                                //     49  !*                   - define BSSavailable
                                                                //     50  !*                   - define Usechipfns
                                                                //     51  !*                   - define Allowvaxnames
                                                                //     52  !*                   - define Allow99cont
                                                                //     53  !*                   - define Realsubscripts
                                                                //     54  !*                   - define AllowI8decl
                                                                //     55  !* modified 13/09/87 - define Evalrealslong
                                                                //     56  !*                   - define AllowI8
                                                                //     57  !* modified 07/09/87 - define Array temps on stack
                                                                //     58  !*
                                                                //     59  %constinteger ICL2900 = 1
                                                                //     60  %constinteger PERQPNX = 2
                                                                //     61  %constinteger ACCENT  = 3
                                                                //     62  %constinteger IBM     = 4
                                                                //     63  !*
                                                                //     64  %constinteger M68000   =6
                                                                //     65  %constinteger Sun3     =6
                                                                //     66  %constinteger INTEL    =7
                                                                //     67  %constinteger LATTICE  =8
                                                                //     68  %constinteger VNS      =9
                                                                //     69  %constinteger GOULD    =10
                                                                //     70  %constinteger WWC      =11
                                                                //     71  %constinteger WCW      =11
                                                                //     72  %constinteger DRS      =11
                                                                //     73  %constinteger AMDAHL   =14
                                                                //     74  %constinteger Sequent  =15
                                                                //     75  %constinteger SPARC    =16
                                                                //     76  %constinteger M88000   =17
                                                                //     77  %constinteger M88K     =17
                                                                //     78  
                                                                //     79  %constinteger i860     =18
                                                                //     80  %constinteger Locust   =SPARC
                                                                //     81  %constinteger Mosquito =M88K
                                                                //     82  %constinteger Tigermoth=M88K
                                                                //     83  %constinteger Aphid    =SPARC
                                                                //     84  %constinteger M88110  =20
                                                                //     85  %constinteger M88100  =21
                                                                //     86  %constinteger M88100forM88110 =22
                                                                //     87  %constinteger RS6000=23
                                                                //     88  %constinteger DRS486   =24
                                                                //     89  %constinteger PTX     =25
                                                                //     90  %constinteger RTMX     =26
                                                                //     91  %constinteger IntelSolaris=27
                                                                //     92  !*
                                                                //     93  %constinteger POSITIVE =  1
                                                                //     94  %constinteger NEGATIVE = -1
                                                                //     95  %constinteger TRUE     =  1
                                                                //     96  %constinteger FALSE    =  0
                                                                //     97  !*
                                                                //     98  %constinteger HOST    = INTEL
                                                                //     99  %constinteger TARGET  = INTEL
                                                                //    100  %constinteger Env     = IntelSolaris
                                                                //    101  !*
                                                                //    102  { UNIX debugger support constants }
                                                                //    103  %constinteger UD UNIMP = 0  { No debugger support provided              }
                                                                //    104  %constinteger UD SDB   = 1  { Support provided for SDB                  }
                                                                //    105  %constinteger UD DBX   = 2  { Support provided for DBX                  }
                                                                //    106  %constinteger UD EDB   = 3  { Support provided for proprietary debugger }
                                                                //    107  !*
                                                                //    108  { UNIX debugger support in this environment }
                                                                //    109  %constinteger UNIXDebug = UD UNIMP    { No debugger support on LOCUST }
                                                                //    110  
                                                                //    111  !*
                                                                //    112  %constinteger UnixType = 5
                                                                //    113  !* Unix type is set to five for LOCUST
                                                                //    114  %constinteger Unix2   = False
                                                                //    115  %constinteger Unix4   = False
                                                                //    116  %constinteger Unix5d4 = False
                                                                //    117  %constinteger Unix5   = True
                                                                //    118  !* Why we have to respecify the Unix type I do not know.  It seems
                                                                //    119  !* illogical, but that is how fntht7.6un.inc works
                                                                //    120  %constinteger STACK DIRECTION = NEGATIVE
                                                                //    121  %constinteger PARAM DIRECTION = POSITIVE
                                                                //    122  !*
                                                                //    123  %constinteger ALIGN8  = (1 << GOULD) ! (1 << SPARC) ! (1 << M88000)
                                                                //    124  !*
                                                                //    125  %constinteger Concept = 0        {}
                                                                //    126  %constinteger NP1     = 1        {only applicable if host/target=Gould}
                                                                //    127  %constinteger Cpu     = Concept  {}
                                                                //    128  %constinteger Goulddivnewbr = 0  { 0 not available  1 available }
                                                                //    129  !*
                                                                //    130  %constinteger Allowvector = 0            { 0  No   1  Yes }
                                                                //    131  %constinteger Allowparallel = 0          { 0  No   1  Yes }
                                                                //    132  %constinteger Emachine      = 4
                                                                //    133  %constinteger Directcomplex = 0  { 0 No (indirect operands)  1 Yes (direct)  }
                                                                //    134  !*
                                                                //    135  %constinteger Array temps on stack = 0   { 0  No   1  Yes }
                                                                //    136  %constinteger AllowI8 = 0                { 0  No   1  Yes }
                                                                //    137  %constinteger Evalrealslong = 0          { 0  No   1  Yes }
                                                                //    138  %constinteger Bytesreversed = 0          { 0  No   1  Yes }
                                                                //    139  %constinteger BSSavailable  = 1          { 0  No   1  Yes }
                                                                //    140  %constinteger Allowvaxnames = 1          { 0  No   1  Yes }
                                                                //    141  %constinteger Allow99cont   = 1          { 0  No   1  Yes }
                                                                //    142  %constinteger Realsubscripts= 1          { 0  No   1  Yes }
                                                                //    143  %constinteger AllowI8decl   = 0          { 0  No   1  Yes }
                                                                //    144  %constinteger AllowI1decl   = 0          { 0  No   1  Yes }
                                                                //    145  %constinteger Allowfind     = 1          { 0  No   1  Yes   definefile,find}
                                                                //    146  %constinteger VAXparam      = 1          { 0  No   1  Yes }
                                                                //    147  %constinteger Bytesigned    = 1          { 0  No   1  Yes }
                                                                //    148  %constinteger FullVAX       = 1          { 0  No   1  Yes }
                                                                //    149  %constinteger Allowgldfns   = 0          { 0  No   1  Yes }
                                                                //    150  %constinteger Allowcase     = 0
                                                                //    151  %constinteger Numoptregs    = 1
                                                                //    152  %constinteger Allowintprc   = 0          {0   No   1  Yes }
                                                                //    153  %constinteger IEEE          = 1
                                                                //    154  %constinteger Scaleopt1     = 1
                                                                //    155  %constinteger Aligndata     = 1 { 1 add filler bytes in common}
                                                                //    156  %constinteger OCScompliant  = 0  { 0 No  1 Yes }
                                                                //    157  !*
                                                                //    158  %constinteger TrueValue     = 1          {usually 1, but x'ff' on PN and MPX}
                                                                //    159  !*
                                                                //    160  %constinteger UTX     = 0        {}
                                                                //    161  %constinteger MPX     = 1        {only applicable if host/target=Gould}
                                                                //    162  %constinteger Opsys   = UTX      {}
                                                                //    163  !*
                                                                //    164  %constinteger M68881     = 1
                                                                //    165  %constinteger Intel287   = 2
                                                                //    166  %constinteger Intel387   = 3
                                                                //    167  %constinteger Weitek     = 4
                                                                //    168  %constinteger Usechipfns = Intel387         {1 if none available}
                                                                //    169  !*
                                                                //    170  %constinteger Fregopt = 1                {0 not available  1 available}
                                                                //    171  !*
                                                                //    172  %constinteger full387 = 0                {applicable only on Intel}
                                                                //    173  !*
                                                                //    174  %constinteger IMP     =  1
                                                                //    175  %constinteger FORTRAN =  2
                                                                //    176  %constinteger CCOMP   = 11
                                                                //    177  %constinteger PASCAL  = 14
                                                                //    178  %constinteger MODULA  = 15
                                                                //    179  
                                                                //    180  %constinteger Compiler = IMP
                                                                //    181  
                                                                //    182  %constinteger ISCOFF = 0
                                                                //    183  %constinteger ISELF = 1
                                                                //    184  %constinteger ISBSD = 2
                                                                //    185  %constinteger ISBSDOMRON = 3
                                                                //    186  %constinteger Object Format = ISBSD
                                                                //    187  %conststring (60) EPCCversion = %c
#line 187 "ft_intel.1.inc"
const _imp_string /*%string(60)*/ EPCCVERSION = _imp_str_literal("EPC C++ Version 2.1.5 (beta)                              \n");
                                                                //    188  "EPC C++ Version 2.1.5 (beta)                              
                                                                //    189  "
                                                                //    190  
                                                                //    191  %conststring (60) EPCCcopyright = %c
#line 191 "ft_intel.1.inc"
const _imp_string /*%string(60)*/ EPCCCOPYRIGHT = _imp_str_literal("Copyright (c) 1992 EPCL. All Rights Reserved. \n");
                                                                //    192  "Copyright (c) 1992 EPCL. All Rights Reserved. 
                                                                //    193  "
                                                                //    194  
                                                                //    195  %endoffile
                                                                //      4  
                                                                //      5  %endoffile
#line 5 "ftnht.inc"
// End of file

void NDIAG(int PC, int LNB, int p3, int p4) {
}
