extern void _MAIN_(void);

extern void _imp_init_files(void);
extern void _imp_initialise(int argc, char **argv);

int main(int argc, char **argv) {
  _imp_initialise(argc, argv);
  _MAIN_();
  return 0;
}
