! Copyright (c) 1987 Edinburgh Portable Compilers Ltd. All Rights Reserved.
!*          01/03/92 - set scaleopt1=1 for RS6000                  ftnht7.14
!*          22/02/92 - Add ISBSDOMRON                              ftnht7.13
!*          14/02/92 - Add RS6000 target for IBM RS6000            ftnht7.12
!*          07/10/90 - Add Object Format for ftnend etc            ftnht7.11
!*          29/11/90 - Add sunf77 for full sun77 compatibility     ftnht7.10
!*          09/09/90 - remove release number and date to vers.inc  ftnht7.9
!*          09/07/90 - updated release number in message           ftnht7.8
!*                   - changed Scaleopt1 to 0
!* modified 05/04/90 - add constinteger for Aligndata              ftnht7.6
!* modified 13/03/90 - add constinteger for Locust                 ftnht7.5
!* modified 18/01/90 - define Allowparallel                        ftnht7.4
!*                   - define Directcomplex
!*                   - define Fregopt
!* modified 12/01/90 - define ALIGN8 for REAL*8 alignment reqmnts  ftnht7.3
!* modified 17/11/89 - define Emachine                             ftnht7.2
!* modified 30/09/89 - define sparc,m88000,i860,paramdirection     ftnht7.1
!* modified 05/09/89 - Add constinteger for modula
!*                   - add constinteger for compiler
!*                   - put reldate in ftnht instead of mfcompile
!* modified 01/05/89 - define TrueValue 
!* modified 25/03/89 - define Allowintprc to govern the legality of
!*                     internal procedures
!* modified 06/01/89 - Remove flag NP2
!* modified 12/12/88 - Set NP2=1 Goulddivnewbr=1 for NP2
!* modified 10/12/88 - NP2 defined as a Boolean
!* modified 25/11/88 - define Goulddivnewbr
!*                   - define Scaleopt1,Allowcase,IEEE
!* modified 06/05/88 - define NP2
!* modified 21/03/88 - define Allowgldfns
!* modified 04/03/88 - more definitions added
!* modified 26/01/88 - define Numoptregs
!* modified 25/01/88 - define Bytesreversed
!*                   - define BSSavailable
!*                   - define Usechipfns
!*                   - define Allowvaxnames
!*                   - define Allow99cont
!*                   - define Realsubscripts
!*                   - define AllowI8decl
!* modified 13/09/87 - define Evalrealslong
!*                   - define AllowI8
!* modified 07/09/87 - define Array temps on stack
!*
%constinteger ICL2900 = 1
%constinteger PERQPNX = 2
%constinteger ACCENT  = 3
%constinteger IBM     = 4
%constinteger VAX     = 5
%constinteger M68000  = 6
%constinteger Sun3    = 6
%constinteger INTEL   = 7
%constinteger LATTICE = 8
%constinteger VNS     = 9
%constinteger GOULD   =10
%constinteger WWC     =11
%constinteger WCW     =11
%constinteger DRS     =11
%constinteger AMDAHL  =14
%constinteger Sequent =15
%constinteger SPARC   =16
%constinteger M88000  =17
%constinteger M88k      =17
%constinteger i860    =18
%constinteger Locust  =19
%constinteger M88110  =20
%constinteger M88100  =21
%constinteger M88100forM88110 =22
%constinteger RS6000=23
%constinteger DRS486   =24
%constinteger PTX     =25
%constinteger MIPS=26
%constinteger PENTIUM=27
!*
%constinteger POSITIVE =  1
%constinteger NEGATIVE = -1
!*
%constinteger HOST    = DRS
%constinteger TARGET  = DRS
%constinteger Env     = DRS486
%constinteger STACK DIRECTION = NEGATIVE
%constinteger PARAM DIRECTION = NEGATIVE
!*
%constinteger ALIGN8  = (1 << GOULD) ! (1 << SPARC) ! (1 << M88000) ! (1<<RS6000)
!*
%constinteger Concept = 0        {}
%constinteger NP1     = 1        {only applicable if host/target=Gould}
%constinteger Cpu     = Concept  {}
%constinteger Goulddivnewbr = 0  { 0 not available  1 available }
!*
%constinteger Allowvector = 0            { 0  No   1  Yes }
%constinteger Allowparallel = 0          { 0  No   1  Yes }
%constinteger Emachine      = 4
%constinteger Directcomplex = 0  { 0 No (indirect operands)  1 Yes (direct)  }
!*
%constinteger Array temps on stack = 0   { 0  No   1  Yes }
%constinteger AllowI8 = 0                { 0  No   1  Yes }
%constinteger Evalrealslong = 0          { 0  No   1  Yes }
%constinteger Bytesreversed = 1          { 0  No   1  Yes }
%constinteger BSSavailable  = 1          { 0  No   1  Yes }
%constinteger Allowvaxnames = 1          { 0  No   1  Yes }
%constinteger Allow99cont   = 1          { 0  No   1  Yes }
%constinteger Realsubscripts= 1          { 0  No   1  Yes }
%constinteger AllowI8decl   = 0          { 0  No   1  Yes }
%constinteger AllowI1decl   = 0          { 0  No   1  Yes }
%constinteger Allowfind     = 1          { 0  No   1  Yes   definefile,find}
%constinteger VAXparam      = 1          { 0  No   1  Yes }
%constinteger Bytesigned    = 1          { 0  No   1  Yes }
%constinteger FullVAX       = 1          { 0  No   1  Yes }
%constinteger Allowgldfns   = 0          { 0  No   1  Yes }
%constinteger Allowcase     = 0
%constinteger Numoptregs    = 1
%constinteger Allowintprc   = 0          {0   No   1  Yes }
%constinteger IEEE          = 1
%constinteger Aligndata     = 0 { 1 add filler bytes in common}
%constinteger SunF77        = 1          {1 = full sun f77 compatibility}
%constinteger OCScompliant  = 0  { 0 No  1 Yes }
%if Target=RS6000 %or Target=Sparc %thenstart
   %constinteger Scaleopt1     = 1
%finishelsestart
   %constinteger Scaleopt1     = 0
%finish
!*
%constinteger TrueValue     = 1          {usually 1, but x'ff' on PN and MPX}
!*
%constinteger UTX     = 0        {}
%constinteger MPX     = 1        {only applicable if host/target=Gould}
%constinteger Opsys   = UTX      {}
!*
%constinteger M68881     = 1
%constinteger Intel287   = 2
%constinteger Intel387   = 3
%constinteger Weitek     = 4
%constinteger Usechipfns = Intel387        {0 if none available}
!*
%constinteger Fregopt = 0                {0 not available  1 available}
!*
%constinteger full387 = 1                {applicable only on Intel}
%constinteger IMP     =  1
%constinteger fortran =  2
%constinteger ccomp   = 11
%constinteger pascal  = 14
%constinteger modula  = 15

%constinteger Compiler = Imp

%constinteger ISCOFF = 0
%constinteger ISELF = 1
%constinteger ISBSD = 2
%constinteger ISBSDOMRON = 3
%constinteger Object Format = ISELF

%endoffile
